/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>*
	DialogUtils.c
	
		R000 07/20/90 Rocky		Creation
		R001 09/27/90 Rocky		Rewrite AdjustPopUpItemRects().
		R002 10/10/90 Rocky		Misc fixes and adjustments.
		R003 11/07/90 dana		fix to SetDoubleText to handle numbers between 0 and .0001
		R004 11/15/90 dana		added routine to allow dimming of items in popup.
		R005 11/20/90 Rocky		add CountPopUpItems().  add check for item
								to be set in AddPopUp().
		R006 11/20/90 dB		moved color picker to deepest screen.
		R007 11/21/90 dB		implemented cleanup for color items.
		R008 11/28/90 Rocky		add DrawPopUpA4, DrawColorItemA4, and UseA4DialogUtils
								make compilable requiring prototypes and checking
								pointer types
		R009 12/11/90 dB		re-fixed SetDoubleText to handle leading - and .  Someone
								appears to have lost this!!!
		R010 12/12/90 dB		SetDoubleText truncated to zero when provided with small
								negative doubles.
		R011 12/27/90 Rocky		Change color items to have variable GetColor dialog prompt.
		R012 01/07/91 Rocky		Add include of ctype.h so we compile checking prototypes.
		R013 02/14/91 dB		Delay takes a pointer in second argument
		R014 02/20/91 dB		New Function SetDoubleTextPlain, GetDoubleTextPlain.
								GetULongTextPlain.		
		R015 03/01/91 Rocky		Change A4 strategy for use when this module is used in code
								resources to not use theDialog->refCon.
		R016 05/02/91 dB		added new routine for handling non resource based popups.
		R017 05/03/91 dB		changed appearance of coloritems.
		R018 05/05/91 dB		added ValidFloat()
		R019 05/06/91 slj		Fixed bug in round-off operation.
		R020 06/06/91 dB		removed stray debugging info in PopUp.
		R021 06/06/91 dB		New routine: GetPopupMenu
		R022 06/07/91 dB		support for empty dynamic menus
								new functions for adding and deleting items from dynamic
								popups.  Various minor cleanups to popup implementation.
		R023 06/08/91 dB		If a popup is empty, don't popup.  Eliminated dynamic
								resizing of menu.
		R024 11/25/91 dB		THINK5 compatibility.
		R025  3/9/93  dB		minor changes...

 *<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

#ifndef	_MacTypes_
#include <MacHeaders>
#endif

#include <Global.h>

#include <DialogUtils.h>

#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#if !THINK4
#include <Picker.h>
#endif

static void __GetA4(void);
#include <SetUpA4.h>

void	DialogUtilsLoad(void);
void	DialogUtilsLoad(void){}	/* To preload or unload segment	*/

typedef struct {
	short		id;			/* ID of menu resource				*/
	MenuHandle	menu;		/* Handle to menu					*/
	short		dlogItem;	/* Dailog Item of PopUp Menu		*/
	short		popUpItem;	/* Currently selected item			*/
	enum		popUpType
				{
					RESOURCE_TYPE, DYNAMIC_TYPE, PERSISTENT_TYPE
				} popUpType;
} PopUpInfo, *PopUpInfoPtr, **PopUpInfoHdl;

typedef struct {
	short		dlogItem;	/* Dailog Item of Color Item		*/
	RGBColor	color;		/* Currently selected item			*/
	Str63		prompt;		/* Prompt for GetColor dialog		*/
} ColorItemInfo, *ColorItemInfoPtr, **ColorItemInfoHdl;

static int		ItemToPopUpNumber(	short			dialogItem	);

static int		ItemToColorNumber(	short			dialogItem	);
									
static void		GetMenuTitle(		MenuHandle		menu,
									StringPtr		title		);
									
static void		DrawPopUpTitle(		DialogPtr		theDialog,
									short			itemNo,
									MenuHandle		menu,
									Boolean			inverted	);
									
static pascal void	DrawPopUpA4(	DialogPtr		theDialog,
									short			itemNo		);
static pascal void	DrawColorItemA4(DialogPtr	theDialog,
									short		itemNo			);
									
char fmtULong[] = "%lu";
char fmtDbl0[]	= "%.0f";
char fmtDbl1[]	= "%.1f";
char fmtDbl2[]	= "%.2f";
char fmtDbl3[]	= "%.3f";  /* R003 */
char fmtDbl4[]	= "%.4f";

static Boolean			gUseA4			= FALSE;
static PopUpInfoHdl		gPopUps			= nil;
static ColorItemInfoHdl	gColorItems		= nil;

/******************************************************************************
 UseA4DialogUtils

		Let's dialog utils know that routines called by toolbox routines such
		as ModalDialog, will have to set up a4 for globals to be accessed.
 ******************************************************************************/
void	UseA4DialogUtils(void)
{
	RememberA4();
	gUseA4 = TRUE;
}

/******************************************************************************
 GetModalFilterProc

		Return ProcPtr to appropriate ModalDialog filter proc for A4 or not.
 ******************************************************************************/
ProcPtr	GetModalFilterProc(void)
{
	if (gUseA4)
		return (ProcPtr)ModalFilterA4;
	else
		return (ProcPtr)ModalFilter;
}

/******************************************************************************
 GetDrawButtonFrameProc

		Return ProcPtr to appropriate DrawButtonFrame draw proc for A4 or not.
 ******************************************************************************/
ProcPtr	GetDrawButtonFrameProc(void)
{
	if (gUseA4)
		return (ProcPtr)DrawButtonFrameA4;
	else
		return (ProcPtr)DrawButtonFrame;
}

#if 0
Redefined in MYTBUtilities.c
/******************************************************************************
 PositionDialog
 
		Center the bounding box of a dialog or alert in the upper third
		of the screen.  This is the preferred location according to the
		Human Interface Guidelines.
 ******************************************************************************/

void	PositionDialog(
	ResType		theType,				/* Resource type, DLOG or ALRT		*/
	short		theID)					/* Res ID of DLOG or ALRT			*/
{
	Handle				theTemplate;	/* Handle to resource template		*/
	register Rect		*theRect;		/* Bounding box of dialog			*/
	register short		left;			/* Left side of centered rect		*/
	register short		top;			/* Top side of centered rect		*/
	
		/* The first field of the resource template for DLOG's and ALRT's */
		/* is its bounding box.  Get a pointer to this rectangle.  This   */
		/* handle dereferencing is safe since the remaining statements in */
		/* this function do not move memory (assignment and simple math). */

	theTemplate = GetResource(theType, theID);
	if (theTemplate == nil)
		return;
		
	theRect = (Rect*) *theTemplate;
	
										/* Center horizontally on screen	*/
	left = (screenBits.bounds.right - (theRect->right - theRect->left)) / 2;

										/* Leave twice as much space below	*/
										/*   as above the rectangle			*/	
	top = (screenBits.bounds.bottom - (theRect->bottom - theRect->top)) / 3;
										/* Don't put rect under menu bar	*/
	top = Max(top, GetMBarHeight() + 1);

	theRect->right += left - theRect->left;
	theRect->left = left;
	theRect->bottom += top - theRect->top;
	theRect->top = top;
}
#endif

/******************************************************************************
 DrawButtonFrameA4

		Version of DrawButtonFrame for use with A4 globals in code resource.
 ******************************************************************************/

pascal void		DrawButtonFrameA4(
	DialogPtr	theDialog,
	short		itemNo)
{
	SetUpA4();
	DrawButtonFrame(theDialog, itemNo);
	RestoreA4();
}

/******************************************************************************
 DrawButtonFrame
 
		User item procedure for drawing frame around default button.
		Have UserItem with rectangle identical to button being framed.
 ******************************************************************************/

pascal void		DrawButtonFrame(
	DialogPtr	theDialog,
	short		itemNo)
{
	short	itemType;
	Handle	itemH;
	Rect	itemBox;
	
	GetDItem(theDialog, ((DialogPeek)theDialog)->aDefItem, &itemType, &itemH, &itemBox);
	InsetRect(&itemBox, -4, -4);
	PenNormal();
	PenSize(3, 3);
	if (((unsigned char)(*(ControlHandle)itemH)->contrlHilite) == 255)
		PenPat((ConstPatternParam)&gray);
	FrameRoundRect(&itemBox, 17, 17);
	PenNormal();
}

/******************************************************************************
 DrawFrame
 
		User item procedure for drawing framing a User Item.
 ******************************************************************************/

pascal void		DrawFrame(
	DialogPtr	theDialog,
	short		itemNo)
{
	short	itemType;
	Handle	itemH;
	Rect	itemBox;
	
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	PenNormal();
	FrameRect(&itemBox);
}
			
/******************************************************************************
 GetULongText

		Given the specified, edit text item, convert to an unsinged long.
 ******************************************************************************/
void	GetULongText(
	DialogPtr		theDialog,
	short			itemNo,
	unsigned long *	value,		/* return value here				*/
	unsigned long	lowerBound,	/* if value less than this, fail	*/
	unsigned long	upperBound)	/* if value greater than this, fail	*/
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	unsigned long	ul;
	char		  * endp;
	
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	GetIText(itemH, theString);
	PtoCstr(theString);
	ul = strtoul((char *)theString, &endp, 10);
	if (*endp != '\0' || ul < lowerBound || ul > upperBound) {
		SysBeep(3);
		SetULongText(theDialog, itemNo, *value);
	} else {
		*value = ul;
	}
}

/******************************************************************************
 GetULongTextPlain

		Given the specified, edit text item, convert to an unsigned long.
 ******************************************************************************/
Boolean	GetULongTextPlain(
	DialogPtr		theDialog,
	short			itemNo,
	unsigned long *	value		/* return value here				*/
)
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	unsigned long	ul;
	char		  * endp;
	
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	GetIText(itemH, theString);
	PtoCstr(theString);
	ul = strtoul((char *)theString, &endp, 10);
	if (*endp != '\0' )
		return TRUE;
	*value = ul;
	return FALSE;
}

/******************************************************************************
 SetULongText

		Given the specified, edit text item, set to to represent value of
		unsigned long.
 ******************************************************************************/
void	SetULongText(
	DialogPtr		theDialog,
	short			itemNo,
	unsigned long	value)
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	double			d;
	char		  * endp;			

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	sprintf((char*)theString, fmtULong, value);
	CtoPstr((char*)theString);
	SetIText(itemH, theString);
}
			
/******************************************************************************
 GetDoubleText

		Given the specified, edit text item, convert to a double.
 ******************************************************************************/
void	GetDoubleText(
	DialogPtr		theDialog,
	short			itemNo,
	double		 *	value,		/* return value here				*/
	double			lowerBound,	/* if value less than this, fail	*/
	double			upperBound)	/* if value greater than this, fail	*/
{
	GetDoubleTextPrec(theDialog, itemNo, value, lowerBound, upperBound, -1);
}
			
/******************************************************************************
 GetDoubleTextPrec

		Given the specified, edit text item, convert to a double, limiting
		the precision to the number of digits as specified.  If the precision
		is specified as -1, then there is no limit to the precision.
 ******************************************************************************/
void	GetDoubleTextPrec(
	DialogPtr		theDialog,
	short			itemNo,
	double		 *	value,		/* return value here				*/
	double			lowerBound,	/* if value less than this, fail	*/
	double			upperBound,	/* if value greater than this, fail	*/
	short			precision)	/* digits past decimal point		*/
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	double			d;
	char		  * endp;			
	double			scratch;
	char		  * fmt;
	Boolean			err = FALSE;
	int				len;

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	GetIText(itemH, theString);
	PtoCstr(theString);

	if (precision > -1 && (endp = strchr((char*)theString, '.')) != NULL) {
		if (precision == 0 || (len = strlen(++endp)) > precision) {
			err = TRUE;
		}
	}
	if (!err) {
		d = strtod((char *)theString, &endp);
		if (*endp != '\0') {	/* invalid float, let's check if it's the beginning of a valid one */
			if(FloatValidate((char *)theString)) {
				if( (0.0 >= lowerBound) && (0 <= upperBound) ) {
					*value = 0.0;
				} else {
					*value = lowerBound;   /* assure that value contains a valid number */
				}
			} else {
				err = TRUE;
			}
		} else if (d < lowerBound || d > upperBound) {
			err = TRUE;
		} else {
			*value = d;
		}
	}
	if (err) {
		SysBeep(3);
		*value = SetDoubleText(theDialog, itemNo, *value);
	}
}
			
/******************************************************************************
 GetDoubleTextPlain

		Given the specified, edit text item, convert to a double.
 ******************************************************************************/
Boolean	GetDoubleTextPlain(
	DialogPtr		theDialog,
	short			itemNo,
	double		 *	value		/* return value here				*/
)	
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	double			d;
	char		  * endp;			

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	GetIText(itemH, theString);
	PtoCstr(theString);
	d = strtod((char *)theString, &endp);
	if (*endp != '\0')
		return TRUE;	/* invalid float, let's check if it's the beginning of a valid one */
	*value = d;
		return FALSE;	/* float is valid */
}
			
/******************************************************************************
 SetDoubleText

		Given the specified, edit text item, set to to represent value of double.
		Return the value of the double to the precision actually represented
		by the text.
 ******************************************************************************/
double	SetDoubleText(
	DialogPtr		theDialog,
	short			itemNo,
	double			value)
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;
	double			d;
	char		  * endp;			
	double			scratch;
	char		  * fmt;
	double			sign;

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
#if 0 /* R003  begin */
	value += .005;	/* Round up to nearest 1/100th */
	value -= modf(value * 100, &scratch) / 100;
	if (modf(value * 10, &scratch) >= .1)
		fmt = fmtDbl2;
	else if (modf(value, &scratch) >= .1)
		fmt = fmtDbl1;
	else
		fmt = fmtDbl0;
#else
	if(value < 0)
	{
		sign = -1.0;
		value = value * -1.0;
	}
	else
		sign = 1.0;
	value += .00005;  /* Round up to nearest 1/10000th */
#if 0 /* R019 begin */
	value -= modf(value*10000, &scratch) / 10000;
#else
	modf(value*10000, &value);
	value /= 10000;
#endif /* R019 end */
	if(modf(value*10000, &scratch) >= .1)
		fmt = fmtDbl4;
	else if(modf(value*1000, &scratch) >= .1)
		fmt = fmtDbl3;
	else if (modf(value * 10, &scratch) >= .1)
		fmt = fmtDbl2;
	else if (modf(value, &scratch) >= .1)
		fmt = fmtDbl1;
	else
		fmt = fmtDbl0;
#endif  /* R003 end */
	sprintf((char*)theString, fmt, sign * value);
	d = strtod((char*)theString, &endp);
	CtoPstr((char *)theString);
	SetIText(itemH, theString);
	
	return d;
}

/******************************************************************************
 SetDoubleTextPlain

		Given the specified, edit text item, set to to represent value of double.
 ******************************************************************************/
void	SetDoubleTextPlain(
	DialogPtr		theDialog,
	short			itemNo,
	double			value)
{
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	Str255			theString;

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	sprintf((char*)theString, "%5.2f", value);
	CtoPstr((char *)theString);
	SetIText(itemH, theString);
}

/******************************************************************************
 PressButton

		Fake a button press
 ******************************************************************************/

Boolean	PressButton(
	DialogPtr	theDialog,
	short		itemNo)
{
	short			itemType;
	ControlHandle	itemH;
	Rect			itemBox;
	long			finalTicks;
	Boolean			isAButton;
	Boolean			isNotHidden;
	Boolean			isEnabled;
	Boolean			handled;
	
	GetDItem(theDialog, itemNo, &itemType, (Handle *)&itemH, &itemBox);

	isAButton	= itemType == ctrlItem + btnCtrl;
	isNotHidden	= itemBox.left < 8192;
	isEnabled	= (unsigned char)(*itemH)->contrlHilite != 255;

	handled	= isAButton && isNotHidden && isEnabled;

	if (handled) {
		/* Make button appear to be pressed */
		HiliteControl(itemH, inThumb);
		Delay(8L, &finalTicks);
		HiliteControl(itemH, 0);
	}

	return handled;
}

/******************************************************************************
 CmdKeyEquivalent

		Check dialog items to see if command key is first char if button item.
 ******************************************************************************/

Boolean	CmdKeyEquivalent(
	DialogPtr		theDialog,
	EventRecord*	theEvent,
	short*			itemHit)
{
	short			itemType;
	ControlHandle	itemH;
	Rect			itemBox;
	Boolean			isAButton;
	char			theChar;
	short			numItems;
	short			itemNo;
	Str255			buttonTitle;
	Boolean			skip;
	Boolean			handled = FALSE;
	
	skip		= (theEvent->modifiers & shiftKey) ? TRUE : FALSE;
		
	theChar		= theEvent->message & charCodeMask;
	theChar		= tolower(theChar);

	numItems	= (*(short *) *((DialogPeek)theDialog)->items) + 1;
	
	for (itemNo = 1; itemNo <= numItems && !handled; itemNo++) {
		GetDItem(theDialog, itemNo, &itemType, (Handle *)&itemH, &itemBox);
		
		isAButton = itemType == ctrlItem + btnCtrl;

		if (isAButton) {
			GetCTitle(itemH, buttonTitle);
			if (theChar == tolower(buttonTitle[1])) {
				if (skip) {
					skip = FALSE;
				} else {
					handled = PressButton(theDialog, itemNo);
					*itemHit = itemNo;
				}
			}
		}
	}
	
	return handled;
}

/******************************************************************************
 ModalFilterA4

		Filter proc does correct return and Command-Dot handling.
 ******************************************************************************/

pascal Boolean	ModalFilterA4(
	DialogPtr		theDialog,
	EventRecord*	theEvent,
	short*			itemHit)
{
	Boolean		result;
	
	SetUpA4();
	result = ModalFilter(theDialog, theEvent, itemHit);
	RestoreA4();
	return(result);
}

/******************************************************************************
 ModalFilter

		Filter proc does correct return and Command-Dot handling.
 ******************************************************************************/

pascal Boolean	ModalFilter(
	DialogPtr		theDialog,
	EventRecord*	theEvent,
	short*			itemHit)
{
	Boolean			handled = FALSE;
	char			theChar;
	short			itemType;
	Handle			itemH;
	Rect			itemBox;
	GrafPtr			oldPort;
	short			itemNo;
	
	GetPort(&oldPort);
	SetPort(theDialog);

	theChar = theEvent->message & charCodeMask;

	switch (theEvent->what) {
		
		case keyDown:
			if ((theEvent->modifiers & cmdKey) && (theChar == '.')) {
				itemNo = Cancel;
				handled = PressButton(theDialog, itemNo);
			} else if (theChar == '\r' || theChar == '\n') {
				itemNo = ((DialogPeek)theDialog)->aDefItem;
				handled = PressButton(theDialog, itemNo);
			} else if (theEvent->modifiers & cmdKey) {
				handled = CmdKeyEquivalent(theDialog, theEvent, &itemNo);
			}
			break;
	}

	SetPort(oldPort);
	
	if (handled)
		*itemHit = itemNo;

	return handled;
}

/******************************************************************************
 AddColorItem

		Add a color picker dialog item to our color dialog system.
		To use this system, call AddColorItem() for as many color
		items as you have in your modal dialog.  Install the
		DrawColorItem() routine as the User Item Proc for all color
		picker user items.  When the item returned from ModalDialog()
		is a color picker user item, call PopUp().  When you are done
		running the dialog, call DisposeColorItems to clean up.
 ******************************************************************************/

void	AddColorItem(
	DialogPtr	theDialog,	/* Dialog contianing PopUp User Item				*/
	short		dialogItem,	/* Item number in modal dialog of color picker item	*/
	RGBColor*	color,		/* Default color									*/
	StringPtr	prompt)		/* Prompt to use in GetColor dialog	R011			*/
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	short		colorNumber;
	long		size;

	if (gColorItems == nil) {
		gColorItems = (ColorItemInfoHdl)NewHandle(sizeof(ColorItemInfo));
		if (gColorItems == nil)
			return;
	} else {
		SetHandleSize((Handle)gColorItems, GetHandleSize((Handle)gColorItems) + sizeof(RGBColor));
	}
	
	GetDItem(theDialog, dialogItem,	&itemType, &itemH,			&itemBox);
	if (!gUseA4)
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawColorItem,	&itemBox);
	else
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawColorItemA4,	&itemBox);

	colorNumber = (GetHandleSize((Handle)gColorItems) / sizeof(ColorItemInfo)) - 1;

	if (prompt == nil || prompt[0] == 0) {
		(*gColorItems)[colorNumber].prompt[0] = 0;
	} else {
		size = prompt[0];
		size = Min(size, 63);
		BlockMove(prompt, (*gColorItems)[colorNumber].prompt, size + 1);
	}
	(*gColorItems)[colorNumber].dlogItem	= dialogItem;
	(*gColorItems)[colorNumber].color		= *color;
}
			
/******************************************************************************
 GetColorItem

		Draw the foreground color item.
 ******************************************************************************/
void	GetColorItem(
	short		itemNo,
	RGBColor*	color)
{
	short		colorNumber;
	
	if (gColorItems == nil)
		return;
		
	colorNumber = ItemToColorNumber(itemNo);
	if (colorNumber == -1) {
		SysBeep(3);
		return;
	}
	
	*color = (*gColorItems)[colorNumber].color;
}
			
/******************************************************************************
 DrawColorItem

		Draw the foreground color item.
 ******************************************************************************/
pascal void	DrawColorItem(
	DialogPtr	theDialog,
	short		itemNo)
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	RGBColor	aColor;
	RGBColor	oldColor;
	short		colorNumber;
	
	colorNumber = ItemToColorNumber(itemNo);
	if (colorNumber == -1) {
		SysBeep(3);
		return;
	}

	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	PenNormal();
	FrameRoundRect(&itemBox, 10, 10);
	InsetRect(&itemBox, 1, 1);
	EraseRoundRect(&itemBox, 10, 10);
	InsetRect(&itemBox, 1, 1);
	aColor = (*gColorItems)[colorNumber].color;
	
	GetForeColor(&oldColor);
	RGBForeColor(&aColor);
	
	PaintRoundRect(&itemBox, 10, 10);
	
	RGBForeColor(&oldColor);
}

/******************************************************************************
 DrawColorItemA4

		Same as DrawColorItem, but called from code resource using A4 globals.
 ******************************************************************************/

static pascal void	DrawColorItemA4(
	DialogPtr	theDialog,	/* The modal dialog PopUp is in				*/
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	SetUpA4();
	DrawColorItem(theDialog, itemNo);
	RestoreA4();
}
								
/******************************************************************************
 PickColorItem

		Use color picker to let user change color.
 ******************************************************************************/
void	PickColorItem(
	DialogPtr	theDialog,
	short		itemNo)
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	Point		where;
	RGBColor	aColor;
	short		colorNumber;
	Str63		aPrompt;
	long		size;
	EventRecord	event;
	Boolean		inButton=TRUE;
	Point		pt;
	
	
	colorNumber = ItemToColorNumber(itemNo);
	if (colorNumber == -1) {
		SysBeep(3);
		return;
	}
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	InvertRect(&itemBox);
	
	while( Button() )
	{
		WaitNextEvent( mDownMask, &event, 0L, 0L );  /* clear out that event */
		GetMouse(&pt);
		if(PtInRect(pt, &itemBox) )
		{
			if(inButton)
				continue;
			inButton = TRUE;
			InvertRect(&itemBox);
		} else
		{
		 	if(!inButton)
		 		continue;
		 	inButton = FALSE;
		 	InvertRect(&itemBox);
		}
	}
	if(!inButton)
		return;
	InvertRect(&itemBox);
	BlockMove((*gColorItems)[colorNumber].prompt, aPrompt, (*gColorItems)[colorNumber].prompt[0] + 1);
	aColor = (*gColorItems)[colorNumber].color;
	where.v = where.h = -1;
	GetColor(where, aPrompt, &aColor, &aColor);
	(*gColorItems)[colorNumber].color = aColor;
	DrawColorItem(theDialog, itemNo);
}
			
/******************************************************************************
 DisposeColorItems

		Clean up color item data.
 ******************************************************************************/
void	DisposeColorItems()
{
	if(gColorItems != nil)
	{
		DisposHandle((Handle)gColorItems);
		gColorItems = nil;
	}
}

/******************************************************************************
 AddPopUp

		Add a PopUp Menu to our little PopUp system here.
		To use this system, call AddPopUp() for as many PopUp
		menus as you have in your modal dialog.  Install the
		DrawPopUp() routine as the User Item Proc for all PopUp
		User Items.  When the item returned from ModalDialog()
		is a PopUp User Item, call PopUp().  When you are done
		running the dialog, call DisposePopUps to clean up.
 ******************************************************************************/

void	AddPopUp(
	DialogPtr	theDialog,	/* Dialog contianing PopUp User Item				*/
	short		theID,		/* Resource of ID of PopUp Menu						*/
	short		dialogItem,	/* Item number in modal dialog of PopUp User Item	*/
	short		popUpItem)	/* Default selected menu item from PopUp Menu		*/
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	short		popUpNumber;
	MenuHandle	menu;

	if (gPopUps == nil) {
		gPopUps = (PopUpInfoHdl)NewHandle(sizeof(PopUpInfo));
		if (gPopUps == nil)
			return;
	} else {
		SetHandleSize((Handle)gPopUps, GetHandleSize((Handle)gPopUps) + sizeof(PopUpInfo));
	}
	
	GetDItem(theDialog, dialogItem,	&itemType, &itemH,		&itemBox);
	
	if (!gUseA4)
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUp,	&itemBox);
	else
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUpA4,	&itemBox);

	popUpNumber = (GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo)) - 1;

	menu = GetMenu(theID);

	if (popUpItem < 1 || popUpItem > CountMItems(menu))	/* R005 */
		popUpItem = 1;

	(*gPopUps)[popUpNumber].id			= theID;
	(*gPopUps)[popUpNumber].dlogItem	= dialogItem;
	(*gPopUps)[popUpNumber].popUpItem	= popUpItem;
	(*gPopUps)[popUpNumber].menu		= menu;
	(*gPopUps)[popUpNumber].popUpType	= RESOURCE_TYPE;

	AdjustPopUpItemRects(theDialog);
}

void	AddPopUpDynamic(	
	DialogPtr		theDialog,
	short			theID,		/* Resource of ID of PopUp Menu						*/
	MenuHandle		theMenu,
	short			dialogItem,
	short			popUpItem	)
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	short		popUpNumber;

	if (gPopUps == nil) {
		gPopUps = (PopUpInfoHdl)NewHandle(sizeof(PopUpInfo));
		if (gPopUps == nil)
			return;
	} else {
		SetHandleSize((Handle)gPopUps, GetHandleSize((Handle)gPopUps) + sizeof(PopUpInfo));
	}
	
	GetDItem(theDialog, dialogItem,	&itemType, &itemH,		&itemBox);
	
	if (!gUseA4)
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUp,	&itemBox);
	else
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUpA4,	&itemBox);

	popUpNumber = (GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo)) - 1;

	if (popUpItem < 1 || popUpItem > CountMItems(theMenu))	/* R005 */
		popUpItem = 1;

	(*gPopUps)[popUpNumber].id			= theID;
	(*gPopUps)[popUpNumber].dlogItem	= dialogItem;
	(*gPopUps)[popUpNumber].popUpItem	= popUpItem;
	(*gPopUps)[popUpNumber].menu		= theMenu;
	(*gPopUps)[popUpNumber].popUpType	= DYNAMIC_TYPE;

	/* AdjustPopUpItemRects(theDialog); */
}

/****
	a persistent popup is one whose menu should not be released between dialog invocations...
 ***/
void
AddPopUpPersistent( DialogPtr	theDialog,
					short		theID,
					MenuHandle	theMenu,
					short		dialogItem,
					short		popUpItem )
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	short		popUpNumber;

	if (gPopUps == nil) {
		gPopUps = (PopUpInfoHdl)NewHandle(sizeof(PopUpInfo));
		if (gPopUps == nil)
			return;
	} else {
		SetHandleSize((Handle)gPopUps, GetHandleSize((Handle)gPopUps) + sizeof(PopUpInfo));
	}
	
	GetDItem(theDialog, dialogItem,	&itemType, &itemH,		&itemBox);
	
	if (!gUseA4)
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUp,	&itemBox);
	else
		SetDItem(theDialog, dialogItem,	 itemType, (Handle)DrawPopUpA4,	&itemBox);

	popUpNumber = (GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo)) - 1;

	if (popUpItem < 1 || popUpItem > CountMItems(theMenu))	/* R005 */
		popUpItem = 1;

	(*gPopUps)[popUpNumber].id			= theID;
	(*gPopUps)[popUpNumber].dlogItem	= dialogItem;
	(*gPopUps)[popUpNumber].popUpItem	= popUpItem;
	(*gPopUps)[popUpNumber].menu		= theMenu;
	(*gPopUps)[popUpNumber].popUpType	= PERSISTENT_TYPE;

	/* AdjustPopUpItemRects(theDialog); */
}

/******************************************************************************
	AppendPopUp:
		append the DYNAMIC popup menu with the string entry provided.
 *****************************************************************************/
void
AppendPopUp( short theItem, Str255 theString )
{
	int			popUpNumber;
	short		popUpItem;
	
	if (gPopUps == nil)
		return;
		
	popUpNumber = ItemToPopUpNumber( theItem);
	if (popUpNumber < 0) {
		SysBeep(3);
		return;
	}
	if( (*gPopUps)[popUpNumber].popUpType != DYNAMIC_TYPE )
		return;	/* can't append to static menus */
	if(theString)
	{
		AppendMenu( (*gPopUps)[popUpNumber].menu, theString );
	}
}

/*****************************************************************************
	DeletePopUpItem:
		only works for DYNAMIC popups
 ****************************************************************************/
void
DeletePopUpItem( short theItem, short popUpItem )
{
	int			popUpNumber;
	
	if (gPopUps == nil)
		return;
		
	popUpNumber = ItemToPopUpNumber( theItem);
	if (popUpNumber < 0) {
		SysBeep(3);
		return;
	}
	if( (*gPopUps)[popUpNumber].popUpType != DYNAMIC_TYPE )
		return;	/* can't append to static menus */
	if( popUpItem <= CountMItems( (*gPopUps)[popUpNumber].menu ) )
	{
		DelMenuItem( (*gPopUps)[popUpNumber].menu, popUpItem );
	}
}

/******************************************************************************
 GetPopUpItem

		Get currently selected item in PopUp.
 ******************************************************************************/

short	GetPopUpItem(
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	int			popUpNumber;
	short		popUpItem;
	
	if (gPopUps == nil)
		return -1;
		
	popUpNumber = ItemToPopUpNumber(itemNo);
	if (popUpNumber < 0) {
		SysBeep(3);
		return -1;
	}
	
	return (*gPopUps)[popUpNumber].popUpItem;
}

/******************************************************************************
 GetPopUpMenu

		Get menu associated with this item.
 ******************************************************************************/
MenuHandle
GetPopUpMenu(
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	int			popUpNumber;
	short		popUpItem;
	
	if (gPopUps == nil)
		return NULL;
		
	popUpNumber = ItemToPopUpNumber(itemNo);
	if (popUpNumber < 0) {
		SysBeep(3);
		return NULL;
	}
	
	return (*gPopUps)[popUpNumber].menu;
}

/******************************************************************************
 DisablePopUpEntry

		disable a menu entry in a popup
 ******************************************************************************/

void	DisablePopUpEntry(
	DialogPtr	theDialog,
	short		itemNo,		/* Dialog item number for PopUp UserItem */
	short		menuItem)	/* the menuItem to disable */
{
	int 		popUpNumber;
	short		popUpItem;
	MenuHandle	menu;
	
	if(gPopUps == nil)
		return;

	popUpNumber = ItemToPopUpNumber(itemNo);
	if(popUpNumber < 0) {
		SysBeep(3);
	}
	menu = (*gPopUps)[popUpNumber].menu;
	if (0 < menuItem && menuItem <= CountMItems(menu))
		DisableItem(menu, menuItem);
}
 
/******************************************************************************
 EnablePopUpEntry

		enable a menu entry in a popup
******************************************************************************/

void	EnablePopUpEntry(
	DialogPtr	theDialog,
	short		itemNo,		/* Dialog item number for PopUp UserItem */
	short		menuItem)	/* the menuItem to disable */
{
	int 		popUpNumber;
	short		popUpItem;
	MenuHandle	menu;
	
	if(gPopUps == nil)
		return;

	popUpNumber = ItemToPopUpNumber(itemNo);
	if(popUpNumber < 0) {
		SysBeep(3);
	}
	menu = (*gPopUps)[popUpNumber].menu;
	if (0 < menuItem && menuItem <= CountMItems(menu))
		EnableItem(menu, menuItem);
}
 
/******************************************************************************
 CountPopUpItems

		return count of number of items in popup
******************************************************************************/

short	CountPopUpItems(
	short		itemNo)		/* Dialog item number for PopUp UserItem */
{
	int 		popUpNumber;
	short		popUpItem;
	MenuHandle	menu;
	
	if(gPopUps == nil)
		return;

	popUpNumber = ItemToPopUpNumber(itemNo);
	if(popUpNumber < 0) {
		SysBeep(3);
	}
	menu = (*gPopUps)[popUpNumber].menu;
	
	return CountMItems(menu);
}

/******************************************************************************
 SetPopUpItem

		Get currently selected item in PopUp.
 ******************************************************************************/

short	SetPopUpItem(
	DialogPtr	theDialog,	/* The modal dialog PopUp is in				*/
	short		itemNo,		/* Dialog item number for PopUp User Item	*/
	short		menuItem)	/* Currently chosen menu item				*/
{
	int			popUpNumber;
	short		popUpItem;
	MenuHandle	menu;
	
	if (gPopUps == nil)
		return -1;
		
	popUpNumber = ItemToPopUpNumber(itemNo);
	if (popUpNumber < 0) {
		SysBeep(3);
		return -1;
	}
	menu = (*gPopUps)[popUpNumber].menu;
	if (0 < menuItem && menuItem <= CountMItems(menu)) {
		(*gPopUps)[popUpNumber].popUpItem = menuItem;
		DrawPopUp(theDialog, itemNo);
	} else
	{
		short		itemType;
		Handle		itemH;
		Rect		itemBox;
		
		GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
		InsetRect(&itemBox, -1, -1);
		InvalRect(&itemBox);
	}
}

/******************************************************************************
 DrawPopUp

		Dialog item draw routine for a PopUp
 ******************************************************************************/

pascal void	DrawPopUp(
	DialogPtr	theDialog,	/* The modal dialog PopUp is in				*/
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	FontInfo	fontInfo;
	Str255		aString;
	int			popUpNumber;
	MenuHandle	menu;
	short		popUpItem;
	RgnHandle	oldClipRgn = thePort->clipRgn, itemRgn;
	
	if (gPopUps == nil)
		return;
		
	PenNormal();
	
	popUpNumber = ItemToPopUpNumber(itemNo);
	if (popUpNumber < 0) {
		SysBeep(3);
		return;
	}
	
	menu		= (*gPopUps)[popUpNumber].menu;
	popUpItem	= (*gPopUps)[popUpNumber].popUpItem;

	DrawPopUpTitle(theDialog, itemNo, menu, FALSE);
	

		/** Get PopUp portion of User Item **/
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	InsetRect(&itemBox, -1, -1);	/* grow the box to erase drop-shadow */
	EraseRect(&itemBox);
	InsetRect(&itemBox, 1, 1);
	if( CountMItems( menu ) > 0 )
	{
		GetFontInfo(&fontInfo);
		if( (*gPopUps)[popUpNumber].popUpType == RESOURCE_TYPE)
		{
			CalcMenuSize(menu);
			itemBox.right = itemBox.left + (**menu).menuWidth + 2;
			SetDItem(theDialog, itemNo,  itemType,  itemH, &itemBox);
		}
	
			/** Draw drop-shadow frame **/
		FrameRect(&itemBox);
		MoveTo(itemBox.left+2,	itemBox.bottom);
		LineTo(itemBox.right,	itemBox.bottom);
		LineTo(itemBox.right,	itemBox.top+2);
	
			/** Draw selected item string **/
		InsetRect(&itemBox, 1, 1);
		EraseRect(&itemBox);
		MoveTo(itemBox.left+fontInfo.widMax, itemBox.top + fontInfo.ascent + fontInfo.leading - 1);
		GetItem(menu, popUpItem, aString);
		itemRgn = NewRgn();
		RectRgn(itemRgn, &itemBox);
		thePort->clipRgn = itemRgn;
		DrawString(aString);
		thePort->clipRgn = oldClipRgn;
		DisposeRgn( itemRgn );
	}
}

/******************************************************************************
 DrawPopUpA4

		Same as DrawPopUp, but called from code resource using A4 globals.
 ******************************************************************************/

static pascal void	DrawPopUpA4(
	DialogPtr	theDialog,	/* The modal dialog PopUp is in				*/
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	SetUpA4();
	DrawPopUp(theDialog, itemNo);
	RestoreA4();
}

/******************************************************************************
 PopUp

		Do PopUp Action
 ******************************************************************************/

void	PopUp(
	DialogPtr	theDialog,	/* The modal dialog PopUp is in				*/
	short		itemNo)		/* Dialog item number for PopUp User Item	*/
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	long		menuChoice;
	short		theID;
	short		theType;
	int			popUpNumber;
	MenuHandle	menu;
	short		item;
	short		id;
	GrafPtr		oldPort;
	
	if (gPopUps == nil)
		return;
		
	popUpNumber = ItemToPopUpNumber(itemNo);
	if (popUpNumber < 0) {
		SysBeep(3);
		return;
	}
	
	menu	= (*gPopUps)[popUpNumber].menu;
	item	= (*gPopUps)[popUpNumber].popUpItem;
	id		= (*gPopUps)[popUpNumber].id;
	
	if(CountMItems(menu) < 1)
		return;

	GetPort(&oldPort);
	SetPort(theDialog);
	
	
	DrawPopUpTitle(theDialog, itemNo, menu, TRUE);
	
		/** Get PopUp portion of User Item **/
	GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
	
	InsetRect(&itemBox, 1, 1);
	LocalToGlobal(&topLeft(itemBox));
				
	InsertMenu(menu, -1);

	CheckItem(menu, item, TRUE);
	
	menuChoice = PopUpMenuSelect(	menu,
									itemBox.top,
									itemBox.left,
									item			);

	CheckItem(menu, item, FALSE);

	DeleteMenu(id);
				
	if (LoWord(menuChoice) > 0) {
		(*gPopUps)[popUpNumber].popUpItem = LoWord(menuChoice);
	}
	
	DrawPopUp(theDialog, itemNo);
	
	SetPort(oldPort);
}

/******************************************************************************
 DisposePopUps

		Clean up our little PopUp system here.
 ******************************************************************************/

void	DisposePopUps()
{
	short	nPopUps;
	int		i;
	OSErr	error;
	
	if (gPopUps != nil) {
		nPopUps = GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo);
		for (i = 0; i < nPopUps; i++) {
			if((*gPopUps)[i].popUpType == RESOURCE_TYPE)
			{
				ReleaseResource((Handle)(*gPopUps)[i].menu);
				error = ResError();
			} else if((*gPopUps)[i].popUpType == DYNAMIC_TYPE)
			{
				DisposeMenu((*gPopUps)[i].menu);
			}
		}
		DisposHandle((Handle)gPopUps);
		gPopUps = nil;
	}
}

/******************************************************************************
 ItemToPopUpNumber

		Get number of PopUp by using the dialog item number.
 ******************************************************************************/

static int	ItemToPopUpNumber(
	short	dialogItem)
{
	register int	nPopUps;
	register int	i;
	
	if (gPopUps != nil) {
		nPopUps = GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo);
		for (i = 0; i < nPopUps; i++) {
			if ( (*gPopUps)[i].dlogItem == dialogItem )
				return i;
		}
	}
	return -1;
}

/******************************************************************************
 ItemToColorNumber

		Get color of color item by using the dialog item number.
 ******************************************************************************/

static int	ItemToColorNumber(
	short		dialogItem)
{
	register int	nColorItems;
	register int	i;
	
	if (gColorItems != nil) {
		nColorItems = GetHandleSize((Handle)gColorItems) / sizeof(ColorItemInfo);
		for (i = 0; i < nColorItems; i++) {
			if ( (*gColorItems)[i].dlogItem == dialogItem )
				return i;
		}
	}
	return -1;
}

/******************************************************************************
 AdjustPopUpItemRects

		Adjust width of all PopUp User Items.
 ******************************************************************************/

void	AdjustPopUpItemRects(
	DialogPtr	theDialog)
{
	short		itemNo;
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	short		popUpNumber;
	int			nPopUps;
	short		itemWidthMax;
	short		titleWidthMax;
	short		totalWidthMax;
	int			popUpIndex;
	short		itemIndex;
	Str255		itemString;
	short		itemWidth;
	FontInfo	fontInfo;
	MenuHandle	menu;
	short		nItems;
	
	if (gPopUps == nil)
		return;
		
	nPopUps = GetHandleSize((Handle)gPopUps) / sizeof(PopUpInfo);

	for (popUpIndex = 0; popUpIndex < nPopUps; popUpIndex++) {
		itemNo =  (*gPopUps)[popUpIndex].dlogItem;
		menu = (*gPopUps)[popUpIndex].menu;
		GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
		CalcMenuSize(menu);
		itemBox.right = itemBox.left + (**menu).menuWidth + 2;
		SetDItem(theDialog, itemNo,  itemType,  itemH, &itemBox);
	}
}

/******************************************************************************
 GetMenuTitle

	Get menu title form menu handle and place in string location provided.
 ******************************************************************************/

static void	GetMenuTitle(
	MenuHandle	menu,
	StringPtr	title)
{
	BlockMove((**menu).menuData, title, (**menu).menuData[0] + 1L);
}

/******************************************************************************
 DrawPopUpTitle

	Draw PopUp menu title.
 ******************************************************************************/

static void	DrawPopUpTitle(
	DialogPtr	theDialog,
	short		itemNo,
	MenuHandle	menu,
	Boolean		inverted)
{
	short		itemType;
	Handle		itemH;
	Rect		itemBox;
	Str255		aString;
	FontInfo	fontInfo;

	GetFontInfo(&fontInfo);
	GetMenuTitle(menu, aString);
	
	if (aString[0] > 1 || (aString[0] == 1 && aString[1] != 0)) {

			/** Get title portion of User Item **/
		GetDItem(theDialog, itemNo, &itemType, &itemH, &itemBox);
		itemBox.right = itemBox.left + 1;
		itemBox.left = itemBox.right - (StringWidth(aString) + ((fontInfo.widMax + 1) / 2));
	
			/** Draw title string **/
		MoveTo(itemBox.left, itemBox.top + fontInfo.ascent + fontInfo.leading);
		itemBox.left -= fontInfo.widMax;
		InsetRect(&itemBox, 1, 1);
		EraseRect(&itemBox);
		DrawString(aString);
		if (inverted)
			InvertRect(&itemBox);
	
	}
}


/*---------------------------------------------------------------------------------------------
	floatValidate:
		does a simple check on a Cstring to verify that the chars are a reasonable float.
 *--------------------------------------------------------------------------------------------*/
Boolean
FloatValidate( char *string )
{
	register char *s = string;
	char c;
	Boolean onedigit = FALSE, oneneg = FALSE, onepoint = FALSE;

	if(!string)
		return FALSE;
	while(*s != '\0')
	{
		c = *s++;
		if(isdigit(c))
			onedigit = TRUE;
		else {
			if(c == '-')
			{
				if(oneneg || onedigit || onepoint)
					return FALSE;
				else
					oneneg = TRUE;
			} else if(c == '.')
			{
				if(onepoint)
					return FALSE;
				onepoint = TRUE;
			} else if( isspace(c) )
			{
				if(oneneg || onedigit || onepoint)
					return FALSE;
			} else
				return FALSE;  /* invalid character */
		}
	}
#if 0
	if(onedigit)
		return TRUE;
	else
		return 0;
#else
	return TRUE;
#endif
}

Boolean IntValidate( char *buf )
{
	char *ptr = buf;
	Boolean firstDigit = FALSE;

	while( *ptr )
	{
		if( !isdigit( *ptr ) )
		{
			if(firstDigit || !isspace( *ptr ))
				return FALSE;
		} else
			firstDigit = TRUE;
		ptr++;
	}
	return TRUE;
}

Boolean ValidDouble( char * s, double *d )
{
	char	*endp;
	
	*d = strtod((char *) s, &endp);					
	if (*endp != '\0')
		return FALSE;
	else
		return TRUE;
}