#include <StandardDefs.h>

#include <GDeviceStuff.h>
#include <WorldStuff.h>
#include <GeneralDefines.h>

#ifndef THINK4
#include <Palettes.h>
#endif

#ifdef applec
#include <Resources.h>
#include <Windows.h>
#include <Script.h>
#include <Palettes.h>
#endif

void
RestoreDeviceCLUTs(void)
{
	GDHandle	aDevice;
	
	for (aDevice = GetDeviceList() ; aDevice != NULL ; aDevice = GetNextDevice(aDevice))
		RestoreDeviceClut(aDevice);
}

void GetWindowGlobalBounds(GrafPtr port, Rect *bounds)
{
	Rect bitsBounds;
	
	*bounds = port->portRect;
	if (port->portBits.rowBytes & 0x8000)
		bitsBounds = (**((CGrafPtr)port)->portPixMap).bounds;
	else
		bitsBounds = port->portBits.bounds;
	OffsetRect(bounds, -bitsBounds.left, -bitsBounds.top);
}

void GetWindowGlobalPos(GrafPtr port, Point *pt)
{
	Rect	bounds;
	
	GetWindowGlobalBounds(port, &bounds);
	pt->h = bounds.left;
	pt->v = bounds.top;
}

void GetMainScreenBounds(Rect *bounds, Boolean subMenuBar)
{
	if (HasColorQD())
		*bounds = (**GetMainDevice()).gdRect;
	else
		/* Do it the old way (only valid when globals are set up) */
#ifdef applec
	*bounds = qd.screenBits.bounds;
#else
	*bounds = screenBits.bounds;
#endif
		
	if (subMenuBar)
		bounds->top += GetMBarHeight();
}

short ScreenCount(void)
{
	GDHandle	gdh;
	short		count = 1;
	
	if (HasColorQD())
		for(gdh = GetDeviceList(), count = 0; gdh != NULL; gdh = GetNextDevice(gdh))
			if(TestDeviceAttribute(gdh, screenDevice) && TestDeviceAttribute(gdh, screenActive))
				count++;
	return(count);
}

void GetScreenBounds(Point globalPt, Rect *bounds, Boolean subMenuBar)
{
	GDHandle	gdh;
	
	/* Default to main screen, if the point is invalid */
	GetMainScreenBounds(bounds, subMenuBar);
	
	if (HasColorQD())
		for(gdh = GetDeviceList(); gdh != NULL; gdh = GetNextDevice(gdh)) {
            if(TestDeviceAttribute(gdh, screenDevice) && TestDeviceAttribute(gdh, screenActive))
            	if (PtInRect(globalPt, &(**gdh).gdRect)) {
            		*bounds = (**gdh).gdRect;
            		if (subMenuBar && TestDeviceAttribute(gdh, mainScreen))
            			bounds->top += GetMBarHeight();
            		break;
            	}
         }
}

void GetDeepestScreenBounds(Rect *bounds, Boolean subMenuBar)
{
	GDHandle	gdh;
	short		maxDepth;
	
	GetMainScreenBounds(bounds, subMenuBar);
		
	maxDepth = 0;
	
	if (HasColorQD())
		for(gdh = GetDeviceList(); gdh != NULL; gdh = GetNextDevice(gdh)) {
            if(TestDeviceAttribute(gdh, screenDevice) && TestDeviceAttribute(gdh, screenActive))
            	if ((**(**gdh).gdPMap).pixelSize > maxDepth) {
            		maxDepth = (**(**gdh).gdPMap).pixelSize;
            		*bounds = (**gdh).gdRect;
            		if (subMenuBar && TestDeviceAttribute(gdh, mainScreen))
            			bounds->top += GetMBarHeight();
            		break;
            	}
         }
}

void CenterRectOnScreen(Rect *rect, short putWhere)
{
	WindowPtr		frontWindow;
	Point			frontWinTopLeft;
	Rect			screenBounds;
	short			left, top;
	
	/* Get bounds of screen on which dialog is to be centered */
	if (putWhere & kOnActiveWinScreen && (frontWindow = FrontWindow()) != NULL) {
		GetWindowGlobalPos(frontWindow, &frontWinTopLeft);
#if 0
		/* Compute global position of top left corner of window */
		if (frontWindow->portBits.rowBytes & 0x8000) {
			frontWinTopLeft.h = frontWindow->portRect.left - (**((CGrafPtr)frontWindow)->portPixMap).bounds.left;
			frontWinTopLeft.v = frontWindow->portRect.top - (**((CGrafPtr)frontWindow)->portPixMap).bounds.top;
		}
		else {
			frontWinTopLeft.h = frontWindow->portRect.left - frontWindow->portBits.bounds.left;
			frontWinTopLeft.v = frontWindow->portRect.top - frontWindow->portBits.bounds.top;
		}
#endif
		GetScreenBounds(frontWinTopLeft, &screenBounds, TRUE);
	}
	else if (putWhere & kOnDeepestScreen)
		GetDeepestScreenBounds(&screenBounds, TRUE);
	else
		GetMainScreenBounds(&screenBounds, TRUE);

	
	left = screenBounds.left 
		+ ((screenBounds.right - screenBounds.left) - (rect->right - rect->left)) / 2;

	top = screenBounds.top 
		+((screenBounds.bottom - screenBounds.top) - (rect->bottom - rect->top)) / 3;
		
	OffsetRect(rect, left - rect->left, top - rect->top);
}

void PositionDialogOn(ResType theType, short theID, short putWhere)
{
	Handle	theTemplate;
	
	if ((theTemplate = GetResource(theType, theID)) == NULL)
		return;
	HLock(theTemplate);
	CenterRectOnScreen((Rect*)*theTemplate, putWhere);
	HUnlock(theTemplate);
}

void PositionDialogOnMain(ResType theType, short theID)
{
	PositionDialogOn(theType, theID, kOnMainScreen);
}

void PositionDialogOnDeepest(ResType theType, short theID)
{
	PositionDialogOn(theType, theID, kOnDeepestScreen);
}

void PositionDialogOnActiveWin(ResType theType, short theID)
{
	PositionDialogOn(theType, theID, kOnActiveWinScreen);
}

OSErr FindDialogPosition(ResType theType, short theID, short putWhere, Point *pos)
{
	Handle			theTemplate;
	Rect			dRect;
	OSErr			err;
	
	/* Get frame for dialog */
	theTemplate = GetResource(theType, theID);
	if ((err = ResError()) != noErr)
		return(err);
	dRect = *(Rect*)*theTemplate;
	ReleaseResource(theTemplate);
	
	CenterRectOnScreen(&dRect, putWhere);
	
	pos->h = dRect.left;
	pos->v = dRect.top;
	return(noErr);
}

Boolean HasScreenDepth(short depth)
{
	GDHandle	gdh;
	
	if (!HasColorQD())
		return(FALSE);
		
	if (depth == 24)
		depth = 32;
	
	for(gdh = GetDeviceList(); gdh != NULL; gdh = GetNextDevice(gdh))
		if(TestDeviceAttribute(gdh, screenDevice) 
		  && TestDeviceAttribute(gdh, screenActive)	
		  && (**(**gdh).gdPMap).pixelSize == depth)
			return(TRUE);
	return(FALSE);
}

/* test whether the specified rect is a valid could be a valid window position... */
Boolean 
ValidWindowRect(Rect *r)
{
	Rect barRect;
	Rect testRect;
	RgnHandle barRgn, testRgn;
	Boolean result = FALSE;
	
	GetMainScreenBounds(&barRect, FALSE);
	barRect.bottom = barRect.top + GetMBarHeight();
	barRgn = NewRgn();
	testRgn = NewRgn();
	RectRgn(barRgn, &barRect);
	DiffRgn(GrayRgn, barRgn, testRgn);
	
	testRect.left = r->left;
	testRect.right = r->left + 5;
	testRect.top = r->top;
	testRect.bottom = r->top + 5;
	InsetRect(&testRect, 2, 2);
	if(RectInRgn( &testRect, testRgn))
		result = TRUE;
	else
	{
		testRect.left = r->right - 5;
		testRect.right = r->right;
		testRect.top = r->top;
		testRect.bottom = r->top + 5;
		InsetRect(&testRect, 2, 2 );
		if(RectInRgn( &testRect, testRgn))
			result = TRUE;
	}
	DisposeRgn(testRgn);
	DisposeRgn(barRgn);
	return result;	
}


/*-----------------------------------CenterWindow BEGIN-----------------------------------*/

void CenterWindow(DialogPtr theDialog, Boolean showWindow, Point theOffset, Rect aroundRect)
{
	short		scrnWidth, scrnHeight, newH, newV;
	Point		absTopLeft, absCenter;

		
	
	scrnWidth = aroundRect.right - aroundRect.left;
	scrnHeight = aroundRect.bottom - aroundRect.top;


	absCenter.h = ((theDialog->portRect.right - theDialog->portRect.left) / 2)
					 + theDialog->portRect.left;
	absCenter.v = ((theDialog->portRect.bottom - theDialog->portRect.top) / 3)
					 + theDialog->portRect.top;
	LocalToGlobal(&absCenter);
		
	absTopLeft.h = theDialog->portRect.left;
	absTopLeft.v = theDialog->portRect.top;
	LocalToGlobal(&absTopLeft);


	newH = ((((scrnWidth/2) - absCenter.h) + absTopLeft.h) + theOffset.h) + aroundRect.left;
	newV = ((((scrnHeight/3) - absCenter.v) + absTopLeft.v) + theOffset.v) + aroundRect.top;
	
	MoveWindow(theDialog, newH, newV, TRUE);
	if (showWindow) ShowWindow(theDialog);

}

/*--------------------------------CenterWindow END-----------------------------------*/



/*---------------------------------ZoomRect BEGIN------------------------------------*/

void ZoomRect(Rect srcRect, Rect dstRect, short sequences)
{
	GrafPtr		savePort, wMgrPort;
	float		tlhSrc=srcRect.left, tlhDst=dstRect.left, tlvSrc=srcRect.top, tlvDst=dstRect.top,
				brhSrc=srcRect.right, brhDst=dstRect.right, brvSrc=srcRect.bottom, brvDst=dstRect.bottom;
	Rect		zoomRect, prevRect={ 0, 0, -1, -1 };
	short		i;
	long		j;
	
					
					
	GetPort(&savePort);
	GetWMgrPort(&wMgrPort);
	SetPort(wMgrPort);
	SetClip(wMgrPort->visRgn);
	PenSize(1,1);
	PenPat(gray);
	PenMode(patXor);
	
	
	tlhDst = (tlhDst - tlhSrc) / sequences;
	tlvDst = (tlvDst - tlvSrc) / sequences;
	brhDst = (brhDst - brhSrc) / sequences;
	brvDst = (brvDst - brvSrc) / sequences;
	
	for (i=0; i<sequences; i++)
	{
		zoomRect.left = (short) tlhSrc;
		zoomRect.top = (short) tlvSrc;
		zoomRect.right = (short) brhSrc;
		zoomRect.bottom = (short) brvSrc;
		
		FrameRect(&zoomRect);
		FrameRect(&prevRect);
		prevRect = zoomRect;
		
		tlhSrc += tlhDst;
		tlvSrc += tlvDst;
		brhSrc += brhDst;
		brvSrc += brvDst;
	}
	FrameRect(&prevRect);


	PenNormal();
	SetPort(savePort);
}

/*---------------------------------ZoomRect END------------------------------------*/



/*-------------------------------LocalRectToGlobalRect BEGIN---------------------------------*/

void LocalRectToGlobalRect(Rect *theRect, GrafPtr rectPort)
{
	Point			tempPoint;
	GrafPtr			savePort;
	
	
	
	GetPort(&savePort);
	
	
	SetPort(rectPort);
	tempPoint.h = (*theRect).left;
	tempPoint.v = (*theRect).top;
	LocalToGlobal(&tempPoint);
	(*theRect).left = tempPoint.h;
	(*theRect).top = tempPoint.v;
	tempPoint.h = (*theRect).right;
	tempPoint.v = (*theRect).bottom;
	LocalToGlobal(&tempPoint);
	(*theRect).right = tempPoint.h;
	(*theRect).bottom = tempPoint.v;


	SetPort(savePort);
}

/*-------------------------------LocalRectToGlobalRect END---------------------------------*/



/*-------------------------------GetGDForPort BEGIN---------------------------------*/

GDHandle GetGDForPort(CGrafPtr ourPort)
{
	GDHandle		theGD;
	Rect			tmpRect, gdRect, pRect;
	Boolean			done=FALSE;
	

	theGD = GetDeviceList();
	
	while (!done)
	{
		gdRect = (**theGD).gdRect;
		pRect = ourPort->portRect;
		LocalRectToGlobalRect(&pRect, (GrafPtr) ourPort);		
		if (SectRect(&pRect, &gdRect, &tmpRect))
			return theGD;
		else
		{
			theGD = GetNextDevice(theGD);
			if (theGD == NULL)
				return theGD;
		}
	}
}

/*-------------------------------GetGDForPort END---------------------------------*/

