/*****************************************************************************/
/*
	File		: Extras.h
	Author		: Steve Johnson
	Description	: This file contains definitions of basic types, macros, traps, etc.
				  which are standardly defined by one compiler but not another. By
				  including this file, it can be assumed that a standard set of 
				  basic types, macros, traps, etc. is defined regardless of the
				  compiler being used.
	
	Revision History
	
	CREATED 91/08/24 SLJ
		- Created empty file, included sections for MPW C, Think C 4.0 and Think C 5.0
	
	R0001 91/08/24 SLJ
		-
*/
/*****************************************************************************/

	
#ifndef _H_StandardDefs
#define _H_StandardDefs

/*
 * Defined here (usually through a standard header) for all platforms:
 *
 * NULL, nil
 * TRUE and FALSE
 * Max and Min
 * VoidFunc, BooleanFunc, ShortFunc, LongFunc
 * p2cstr, c2pstr, PtoCstr, CtoPstr
 * OffsetOf
 */

/*****************************************************************************/
/*                          	  Think C 4.0               				 */
/*****************************************************************************/
#if THINK_C == 1
#define THINK4	1
#include <MacTypes.h>
#include <Global.h>
#include <Pascal.h>
#define nil (0L)
#define c2pstr(s) CtoPstr(s)
#define p2cstr(s) PtoCstr(s)
#endif

/*****************************************************************************/
/*                          	  Think C 5.0               				 */
/*****************************************************************************/
#if THINK_C == 5
#define THINK5 1
#include <Types.h>
#include <Think.h>
#include <Global.h>
#include <Pascal.h>
#ifdef STDDEFS_SANE_MATH
#include <sane.h>
#else
#include <math.h>
#endif
#undef MAXINT
#undef MAXLONG
#include <values.h>
#endif

/*****************************************************************************/
/*             Macintosh Programmer's Workshop C (all versions)              */
/*****************************************************************************/
#ifdef applec
#include <Types.h>
#include <Strings.h>
#define FALSE			0
#define TRUE			!FALSE
#define	Max(x, y)		((x) > (y) ? (x) : (y))
#define	Min(x, y)		((x) < (y) ? (x) : (y))
#define CtoPstr(s) c2pstr(s)
#define PtoCstr(s) p2cstr(s)
typedef		void (*VoidFunc)();			/* Ptr to a function returning void	*/
typedef		Boolean (*BooleanFunc)();	/* Ptr to a Boolean function		*/
typedef		long (*LongFunc)();			/* Ptr to a function returning long	*/
#endif

/*****************************************************************************/
/*                   Common definitions for all compilers                    */
/*****************************************************************************/

#ifdef applec
typedef short (*ShortFunc)();			/* Ptr to a function returning short */
#else
typedef short (*ShortFunc)(...);
#endif

#define OffsetOf(field, structType) ((long)&((structType *)0L)->field)

#endif /* _H_StandardDefs */

