/*==========================================================*/
/*
	File	: WorldStuff.h
	Author	: steve johnson
	
	Routines and defines to test for the existing of system configuration

	Revision History
	
	R0000 12/08/90 SLJ
	-	created
	
	R0001 01/30/91 JW
	-	added other useful trap values
	
	R0002 02/10/91 SLJ
	-	added HasSpecialFolders function (see FolderStuff.[ch])
	
*/
/*==========================================================*/

#include <StandardDefs.h>

#ifdef THINK4
#include <SubProjects_Config.h>
#endif

#include <WorldStuff.h>
#include <MacDefines.h>

#ifdef THINK4
#include <SubProjects_Config.h>
#include <MemoryMgr.h>
#include <OSUtil.h>
#include <EventMgr.h>
#endif

#ifdef applec
#include <Traps.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <SysEqu.h>
#include <Memory.h>
#endif

#ifdef THINK4
#include <Gestalt.h>
#else
#include <GestaltEqu.h>
#endif

#define QD32_TRAP	0xab03

#if 0	/* These don't work, at least in a driver */

/* The following 3 routines are taken from IM Volume 6, pages 3-8,9 */

static short NumToolboxTraps(void)
{
	if (NGetTrapAddress(_InitGraf, ToolTrap) == NGetTrapAddress(0xaa6e, ToolTrap))
	 	return(0x200);
	 else
	 	return(0x400);
}

static TrapType GetTrapType(short theTrap)
{
	if (theTrap & 0x0800)
		return(ToolTrap);
	else
		return(OSTrap);
}

Boolean NTrapAvailable(short theTrap)
{
	TrapType	tType;
	
	tType = GetTrapType(theTrap);
	if (tType == ToolTrap) {
		theTrap &= 0x07ff;
		if (theTrap >= NumToolboxTraps())
			theTrap = _Unimplemented;
	}
	return(NGetTrapAddress(theTrap, tType) != NGetTrapAddress(_Unimplemented, ToolTrap));
}


static TrapType GetTrapType(short theTrap)
{
	if (theTrap & 0x0800)
		return(ToolTrap);
	else
		return(OSTrap);
}

#endif

#if !defined(LIBRARIAN) && !defined(SHOWPLACE)
void SetStack(long stackSize)
{
	Ptr		oldApplLimit, newApplLimit;
	
#ifdef THINK_C
	if ((oldApplLimit = GetApplLimit()) > (newApplLimit = CurStackBase - stackSize))
		SetApplLimit(newApplLimit);	
#else
	if ((oldApplLimit = GetApplLimit()) > (newApplLimit = *(Ptr *)CurStackBase - stackSize))
		SetApplLimit(newApplLimit);	
#endif
}
#endif

long FreeStackSpace(void)
{
	Ptr stackPtr;
	asm {
		move.l		sp, stackPtr;
	}
	return(stackPtr - GetApplLimit());
}

Boolean NTrapAvailable( short theTrap, short itsType)
{
	return ( GetTrapAddress(_Unimplemented) != NGetTrapAddress(theTrap, itsType) );
}


Boolean CanAllocateTempMem(void)
{
	long	response;
	
	if (HasGestalt() && Gestalt(gestaltOSAttr, &response) == noErr)
		return(response & (1 << gestaltTempMemSupport));
	else
		return(NTrapAvailable(_OSDispatch, ToolTrap));
}

Boolean CanAllocateTempMemAsReal(void)
{
	long	response;
	
	if (HasGestalt() && Gestalt(gestaltOSAttr, &response) == noErr)
		return(response & (1 << gestaltTempMemSupport)
		  && response & (1 << gestaltRealTempMemory)
		  && response & (1 << gestaltTempMemTracked));
	else
		return(NTrapAvailable(_OSDispatch, ToolTrap));
}

Boolean CanRunMultipleProcesses(void)
{
	long	response;
	
	if (HasGestalt() && Gestalt(gestaltOSAttr, &response) == noErr)
		return(response & (1 << gestaltLaunchCanReturn));
	else
		return(NTrapAvailable(_OSDispatch, ToolTrap));
}

Boolean HasGestalt(void)
{
	return(NTrapAvailable(_Gestalt, ToolTrap));
}

Boolean HasColorQD(void)
{
	SysEnvRec	theWorld;
	long		response;
	
	if (HasGestalt() && Gestalt(gestaltQuickdrawVersion, &response) == noErr)
		return((response & (gestalt8BitQD | gestalt32BitQD)) != 0);
	else
		return(SysEnvirons(1, &theWorld) == noErr &&  theWorld.hasColorQD);
}

Boolean Has32BitQD(void)
{
	long		response;
	
	if (HasGestalt() && Gestalt(gestaltQuickdrawVersion, &response) == noErr)
		return((response & gestalt32BitQD) != 0);
	else
		return(HasColorQD() && NTrapAvailable(QD32_TRAP, ToolTrap));
}

Boolean HasSpecialFolders(void)
{
	long response;
	
	return (HasGestalt() && Gestalt(gestaltFindFolderAttr, &response) == noErr
	  && response  & (1 << gestaltFindFolderPresent));
}

short GetSystemVersion(void)
{
	SysEnvRec	environ;
	long		response;
	
	if (HasGestalt() && Gestalt(gestaltSystemVersion, &response) == noErr)
		return(response);
	else if (SysEnvirons(1, &environ) == noErr)
		return(environ.systemVersion);
	else
		return(0);
}

static char sMMUMode;

void MMUMode32(void)
{
	sMMUMode = true32b;
	SwapMMUMode(&sMMUMode);
}

void MMUModeBack(void)
{
	SwapMMUMode(&sMMUMode);
}

Boolean IsCmdPeriod(void)
{
	register 	EvQElPtr	eventQPtr;
	EventRecord				theEvent;

	/* search for a cmd-'.' in event queue */
	
	if ( eventQPtr = (EvQElPtr)( GetEvQHdr() )->qHead )
		do {
			if ( eventQPtr->evtQWhat == keyDown || eventQPtr->evtQWhat == autoKey ) {
				if ( (char) (eventQPtr->evtQMessage & charCodeMask) == '.' &&
						( eventQPtr->evtQModifiers &  cmdKey )
				) {
				
					/* remove all keyboard events upto and including the cmd-'.' */
					
					while ( GetOSEvent(keyDownMask|autoKeyMask,&theEvent) )
						if ( (char) (theEvent.message & charCodeMask) == '.' &&
								( theEvent.modifiers &  cmdKey )
						) {
							return TRUE;
						}
				}
			}
		} while ( eventQPtr = (EvQElPtr)eventQPtr->qLink );

	return false;
}

Boolean AmBackgroundOnly(void)
{
	short 		**sizeRes;
	Boolean		result;
	
	sizeRes = (short **)GetResource('SIZE', 0);
	if (sizeRes == NULL)
		sizeRes = (short **)GetResource('SIZE', -1);
	if (sizeRes == NULL)
		return(FALSE);
	result = (**sizeRes & 0x0400) != 0;
	ReleaseResource((Handle)sizeRes);
	return(result);
}

#define SYS7PM_BITS ((1 << gestaltLaunchCanReturn) | (1 << gestaltLaunchFullFileSpec) | (1 << gestaltLaunchControl))					

Boolean HasNewProcessMgr(void)
{
	long gestaltResult;
	short state = -1;
	
	if (state < 0)
		if (HasGestalt() && Gestalt(gestaltOSAttr, &gestaltResult) == noErr
		  && (gestaltResult & SYS7PM_BITS) == SYS7PM_BITS)
			state = 1;
		else
			state = 0;
	return(state);
}
	


