/*	rip.h	1.29	93/04/08	*/

/*---------------------------------------------------------------------
 *
 * Copyright (c) 1988 Pixar.   All rights reserved.   This program
 * or documentation contains  proprietary confidential information
 * and trade secrets of Pixar.  Reverse engineering of object code
 * is prohibited.   Use of copyright  notice is precautionary  and
 * does not imply publication.
 *
 *                    RESTRICTED RIGHTS NOTICE
 *
 * Use,  duplication,  or disclosure by the Government is subject to
 * restrictions as set forth in subdivision (b)(3)(ii) of the Rights
 * in Technical Data and Computer Software clause at 252.227-7013.
 *
 * Pixar
 * 3240 Kerner Blvd.
 * San Rafael, CA  94901-4840
 *
 *---------------------------------------------------------------------
 */

#ifndef _RIP_
#define	_RIP_
/*
 * RenderMan Interface Protocol Definitions.
 */
#include <stdio.h>

#if !defined(applec) && !defined(THINK_C)
#include <sys/types.h>
#endif

#include "ri.h"
#include "ripcodes.h"

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

typedef int RtError;

typedef	struct ripbuf {
	struct	ripbuf *next, *prev;	/* two way street */
	short	off;			/* offset to valid data */
	short	space;			/* space left in in buffer */
	unsigned char	*_ptr;		/* pointer to open space in buffer */
	unsigned char	*buf;		/* data buffer */
} RIPBUF;

typedef	struct ripfile {
	RIPBUF	*_head, *_tail;	/* list of buffers (must be first) */
	RIPBUF	*_cur;		/* current one in use */
	short	_cnt;		/* total # of characters */
	short	_bufsize;	/* size of buffer */
	short	_flag;		/* stdio emulation (sort of) */
#define	RIP_IOREAD	01	/* stream open for reading */
#define	RIP_IOWRT	02	/* stream open for writing */
#define	RIP_MYBUF	010	/* we allocated i/o buffer */
#define	RIP_IOEOF	020	/* end-of-file encountered */
#define	RIP_IOERR	040	/* error on i/o stream */
#define	RIP_BINARY	0100	/* binary format output */
	enum { EMPTY, HEADER, BODY } _fsm;	/* header state machine */
#ifdef	MSDOS
	FILE	*_file;		/* fopen file descriptor */
#else
	char	_file;		/* open file descriptor */
#endif
	RIPBUF	*_free;		/* free list of buffers */
} RIPSTREAM;

#ifndef NULL
#define	NULL	0
#endif

struct ripiobuf {
	RIPSTREAM iob;
	int	protected;
	RIPSTREAM	*outputside;
};

#define rip_putc(x,p)	(--(p)->_cur->space >=0? \
    ((p)->_cnt++, (int)(*(p)->_cur->_ptr++=(unsigned)(x))): \
      RIPFlsbuf((unsigned)(x),p))
#define	rip_getc(p)	(--(p)->_cnt >= 0 ? \
    ((unsigned char)*(p)->_ptr++): RIPFilbuf(p))
#define	rip_fileno(p)	((p)->_file)
#define	rip_eof(p)	((p)->_flag & RIP_IOEOF)
#define	rip_error(p)	((p)->_flag & RIP_IOERR)
#define	rip_clrerr(p)	((p)->_flag &= ~(RIP_IOERR|RIP_IOEOF))
#define	rip_isbinary(p)	((p)->_flag & RIP_BINARY)

extern	RIPSTREAM *RIOutputStream;
extern	RIPSTREAM *RIInputStream;

#ifdef INVALID	/* this symbol is define on mac in sane.h  */
#undef INVALID
#endif

typedef	enum {
	INVALID,
	RI_ERROR_IGNORE,
	RI_ERROR_PRINT,
	RI_ERROR_ABORT,
	RI_ERROR_HANDLER
} RtErrorModes;

/* work around type redefinition bogosity... */
typedef	struct { RtMatrix mat; } RiMatrixType;

extern	RtToken RI_BOX, RI_TRIANGLE, RI_GAUSSIAN, RI_SINC, RI_BESSEL;
extern	RtToken	RI_POWER, RI_BSPLINE, RI_HERMITE, RI_BEZIER, RI_CATROM;

typedef	enum {
	RIPFORMAT_UNDEF,
	RIPFORMAT_ASCII,
	RIPFORMAT_BINARY
} RIPFormat;
extern	RIPFormat RipFormat;

extern	char *RiServer;
extern	float RIPVersion;

/* This was moved here from token.c so that the Mac Code Resource code
   can allocate a 'struct hashbucket' as part of a larger structure.  The
   coded that does that needs to know what a 'struct hashbucket' is.
 */
#define	BUCKETS	128
struct hash {
	struct hash *next;	/* linked list */
	char magic[4];		/* magic string identifying a hash entry */
	short	len;		/* string length */
	short	id;		/* unique id for binary encoding */
	char	str[1];		/* variable length */
};
struct	hashbucket {
	struct hash *next;
};

#ifdef	__STDC__
extern	RtError RIPRequest(char *fmt, int code, ...);
extern	RtError RIPRequest1(char *fmt, char *request, ...);
extern	RtError RIPPrintf(RIPSTREAM *s, char *fmt, ...);
extern	char *RIFilterFunc2Name(RtFilterFunc filterfunc);
extern	RtInt RIPGetLightHandleId(RtLightHandle handle);
extern	RtInt RIPGetObjectHandleId(RtObjectHandle handle);
extern	RtError RIPException(void);
extern	RtError RIPPushHandleContext(void);
extern	RtError RIPPopHandleContext(void);
extern	RtLightHandle RIPCreateLightHandle(void);
extern	RtObjectHandle RIPCreateObjectHandle(void);
extern	RtVoid RIPInitPreDefinedItems(void);
extern	RtVoid RIPErrorHandler(RtVoid (*handler)(RtInt, RtInt, char *));
extern	RtVoid XcptHandler(RtInt code, RtInt severity, char *msg);
extern	RtVoid RIPExit(int code);
extern	int RIPFlush(RIPSTREAM *iop);
extern	RtVoid RIPNewBuf(RIPSTREAM *iop);
extern	RtVoid RiFlush(void);
extern	RIPSTREAM *RIPInit(char *server, RIPFormat format);
extern	RtVoid RIPInitTokenStore(void);
extern	RtVoid RIPInitParamStore(void);
extern	RtVoid RIPInitRequestEncoding(void);
extern	RtVoid RIPStamp(RIPSTREAM *stream, char *string);
extern	int RIPTokenID(char *name, int len, int *dodef);
extern	RtVoid RiVArchiveRecord(RtToken type, char *format, va_list vap);
extern	RtVoid RiVMotionBegin(RtInt n, va_list vap);
#else
extern	RtError RIPRequest();
extern	RtError RIPPrintf();
extern	char *RIFilterFunc2Name();
extern	RtInt RIPGetLightHandleId();
extern	RtInt RIPGetObjectHandleId();
extern	RtError RIPException();
extern	RtError RIPPushHandleContext();
extern	RtError RIPPopHandleContext();
extern	RtLightHandle RIPCreateLightHandle();
extern	RtObjectHandle RIPCreateObjectHandle();
extern	RtVoid RIPInitPreDefinedItems();
extern	RtVoid RIPErrorHandler();
extern	RtVoid XcptHandler();
extern	RtVoid RIPExit();
extern	int RIPFlush();
extern	RtVoid RIPNewBuf();
extern	RtVoid RiFlush();
extern	RIPSTREAM *RIPInit();
extern	RtVoid RIPInitTokenStore();
extern	RtVoid RIPInitParamStore();
extern	RtVoid RIPInitRequestEncoding();
extern	RtVoid RIPStamp();
extern	int RIPTokenID();
extern	RtVoid RiVArchiveRecord();
extern	RtVoid RiVMotionBegin();
#endif	/* __STDC__ */

#if defined(THINK_C) || defined(applec)
extern	double j1(double arg);
extern	double y1(double arg);
typedef long (*RibClientFlushProc)(char *, long);
#endif

#ifdef MAC_CODE_RSRC
extern	void MacCodeRsrcLoadRequestNames(void);
extern	void MacCodeRsrcKillRequestNames(void);
extern	void MacCodeRsrcTokenInit(void);
#endif

#include "rip_pid.h"

#endif /* _RIP_ */
