#include <SetupA4.h>
#include <Components.h>
#include "RibGenPlugIn_common.h"
#include "RibGenPlugIn.h"
#include <RsrcMapStuff.h>
#include <LibRibStandardFace.h>
#include <GDeviceStuff.h>
#include <GeneralDefines.h>

#ifdef COMPONENT_VERSION
#define kComponentVersion			COMPONENT_VERSION
#else
#define kComponentVersion			0x00010000
#endif

static float sLastProgress;

#ifndef RIBGENPLUGIN_TEST

typedef struct {
	ComponentInstance			self;
	RsrcMapH					rsrcMap;
	short						appResRef;
	long						appA4;
	RibGen_Callbacks			callbacks;
} Storage_Struct, **Storage_H;

static Storage_H	sStorage = NULL;

static pascal ComponentResult ccComponentOpen(ComponentInstance self)
{
	short				saveRes;
	OSErr				err;
	short				resRef;
	
	/* Copy the calling app's QD globals to our local copy */
	asm	{
			MOVE.L	(A5),A0 		; Get address of real quickdraw globals
			SUB.L	#126,A0 		; Move to start
			LEA 	randSeed,A1 	; Get address of local copy
			MOVE.W	#64,D0			; Globals are 65 words long
		@1	MOVE.W	(A0)+,(A1)+ 	; Copy a word
			DBF 	D0,@1			; Move to next word
	}
				
	if ((sStorage = (Storage_H)NewHandleClear(sizeof(Storage_Struct))) == NULL)
		return memFullErr;
		
	saveRes = CurResFile();
	resRef = OpenComponentResFile((Component)self);
	if ((err = ResError()) != noErr) {
		DisposHandle((Handle)sStorage);
		return err;
	}
	if (((**sStorage).rsrcMap = FindRsrcMap(resRef)) == NULL) {
		CloseComponentResFile(resRef);
		DisposHandle((Handle)sStorage);
		UseResFile(saveRes);
		return paramErr;
	}
	UseResFile((**(**sStorage).rsrcMap).fRefNum);
	
	MoveHHi((Handle)sStorage);
	HLock((Handle)sStorage);
	(**sStorage).self = self;
	if ((err = OpenPlugIn()) != noErr) {
		UnloadA4Seg(NULL);
		
		CloseComponentResFile(resRef);
		
		HUnlock((Handle)sStorage);
		DisposHandle((Handle)sStorage);
		sStorage = NULL;

		SetComponentInstanceStorage(self, NULL);
	
		UseResFile(saveRes);
		
		return err;
	}
	
	SetComponentInstanceStorage(self, (Handle)sStorage);
	
	UseResFile(saveRes);
	
	RemoveRsrcMap((**sStorage).rsrcMap);
		
	return err;
}

static pascal ComponentResult ccComponentClose(ComponentInstance self)
{
	short	saveRes;
	
	sStorage = (Storage_H) GetComponentInstanceStorage(self);
	
	saveRes = CurResFile();

	InsertRsrcMap((**sStorage).rsrcMap, NULL, TRUE);

	ClosePlugIn();
	
	UnloadA4Seg(NULL);
	
	CloseComponentResFile((**(**sStorage).rsrcMap).fRefNum);
	UseResFile(saveRes);
	
	HUnlock((Handle)sStorage);
	DisposHandle((Handle)sStorage);
	SetComponentInstanceStorage(self, NULL);
	
	
	return noErr;
}

static pascal ComponentResult ccComponentCanDo(short selector)
{
	return (
	  (selector >= kComponentVersionSelect && selector <= kComponentOpenSelect) ||
	  (selector >= kRibGen_FirstSelector && selector <= kRibGen_LastSelector)
	  )? 1 : 0;
}

static pascal ComponentResult ccComponentVersion(ComponentInstance self)
{
	return (ComponentResult)kComponentVersion;
}
#endif /* RIBGENPLUGIN_TEST */

/************************* RIBGEN PLUGIN ROUTINES *************************/

pascal ComponentResult ccInit(Handle plugInData, RibGen_CallbacksPtr callbacks)
{
#ifndef RIBGENPLUGIN_TEST
	(**sStorage).callbacks = *callbacks;
#endif
	return InitPlugIn(plugInData);
}

pascal ComponentResult ccModify(Handle plugInData, Handle object)
{
	return ModifyPlugInObject(plugInData, object);
}

	static RtVoid
	ErrorHandler(RtInt code, RtInt severity, char *message)
	{
		HandleRibGenError(code, severity, message);
	}

	static Handle sRibH;
	
	static long
	ToHandleFlushProc(char * buf, long count)
	{
		Munger(sRibH, GetHandleSize(sRibH), NULL, 0, buf, count);
		if (MemError())
			ErrorHandler(RIE_NOMEM, RIE_SEVERE, "Ran out of memory trying to generate geometry.");
		else
			RibGenPlugIn_Progress(-1);
		return count;
	}

pascal ComponentResult ccGenerate(Handle plugInData, Handle object, float time, Handle rib)
{
	OSErr				err = noErr;
	short				n;
	ComponentResult		result;
	
	if (object == NULL || rib == NULL)
		return RG_BAD_PARAM;
	
	if (LibRibStandardFace_OpenInstance() != noErr)
		return RG_COMPONENT_ERR;

	SetHandleSize(rib, 0);
	if (MemError() != noErr)
		return RG_MEM_FULL;
	sRibH = rib;
	RIPErrorHandler(ErrorHandler);
	RIPFlushProc(ToHandleFlushProc);

	sLastProgress = 0.0;
	RiBegin(NULL);
	result = GeneratePlugInRib(plugInData, object, time);
	RiEnd();
	
	LibRibStandardFace_CloseInstance();
	
	return result;
}

	static short sRibFileRef;
	
	static long
	ToFileFlushProc(char * buf, long count)
	{
		char *cp;
		
		for (cp = buf ; cp < &buf[count] ; cp++)
			if (*cp == '\n')
				*cp = '\r';
		if (FSWrite(sRibFileRef, &count, buf) != noErr)
			ErrorHandler(RIE_DISKFULL, RIE_SEVERE, "Ran out of disk space trying to generate geometry.");
		else
			RibGenPlugIn_Progress(-1);
		return count;
	}

pascal ComponentResult ccGenToFileRef(Handle plugInData, Handle object, float time, short fileRef)
{
	ComponentInstance	ci;
	ComponentResult		result;
	
	if (object == NULL || fileRef == 0)
		return RG_BAD_PARAM;
	
	if (LibRibStandardFace_OpenInstance() != noErr)
		return RG_COMPONENT_ERR;

	sRibFileRef = fileRef;
	RIPErrorHandler(ErrorHandler);
	RIPFlushProc(ToFileFlushProc);
	
	sLastProgress = 0.0;
	RiBegin(NULL);
	result = GeneratePlugInRib(plugInData, object, time);
	RiEnd();
	
	LibRibStandardFace_CloseInstance();
	
	return result;
}

pascal ComponentResult ccGenToFile(Handle plugInData, Handle object, float time, FSSpec *file)
{
	ComponentResult		err;
	short				ref;
	
	HCreate(file->vRefNum, file->parID, file->name, 'RINI', 'TEXT');
	if (HOpen(file->vRefNum, file->parID, file->name, fsWrPerm, &ref) != noErr)
		return RG_FOPEN_ERR;
	SetEOF(ref, 0);
	err = ccGenToFileRef(plugInData, object, time, ref);
	FSClose(ref);
	return err;
}

pascal ComponentResult ccCleanup(Handle plugInData)
{
	CleanupPlugIn(plugInData);
	return RG_NO_ERR;
}

/************************* CALLBACKS *************************/

void RibGenPlugIn_Progress(float progress)
{
#ifndef RIBGENPLUGIN_TEST
	void (*callback)(float);
	long saveA4, appA4;
	short saveResRef;
	
	if ((callback = (**sStorage).callbacks.cbRibGenProgress) != NULL) {
		if (progress < 0)
			progress = sLastProgress;
		else
			sLastProgress = progress;
		saveResRef = CurResFile();
		UseResFile((**sStorage).appResRef);
		appA4 = (**sStorage).appA4;
		asm {
			move.l	a4, saveA4
			move.l	appA4, a4
		}
		(callback)(progress);
		asm {
			move.l	saveA4, a4
		}
		UseResFile(saveResRef);
	}
#endif	
}

/************************* RIBGEN PLUGIN MAIN *************************/

#ifndef RIBGENPLUGIN_TEST
pascal ComponentResult main(ComponentParameters* params, Storage_H storage)
{
	ComponentResult		result = badComponentSelector;
	long 				ayFour;
	
	asm {
		move.l	a4, ayFour
	}
	
	RememberA0();
	SetUpA4();
	
	switch (params->what) {
	
		case kComponentOpenSelect:
			result = CallComponentFunction(params, ccComponentOpen); break;
		case kComponentCloseSelect:
			result = CallComponentFunction(params, ccComponentClose); break;
		case kComponentCanDoSelect:
			result = CallComponentFunction(params, ccComponentCanDo); break;
		case kComponentVersionSelect:
			result = CallComponentFunction(params, ccComponentVersion); break;
			
		default:

			sStorage = storage;
				
			(**storage).appResRef = CurResFile();
			(**storage).appA4 = ayFour;
			InsertRsrcMap((**sStorage).rsrcMap, NULL, TRUE);

			switch (params->what) {
				case kRibGen_Init:
					result = CallComponentFunction(params, ccInit); break;
				case kRibGen_Modify:
					result = CallComponentFunction(params, ccModify); break;
				case kRibGen_Generate:
					result = CallComponentFunction(params, ccGenerate); break;
				case kRibGen_GenToFile:
					result = CallComponentFunction(params, ccGenToFile); break;
				case kRibGen_GenToFileRef:
					result = CallComponentFunction(params, ccGenToFileRef); break;
				case kRibGen_Cleanup:
					result = CallComponentFunction(params, ccCleanup); break;
			}

			UseResFile((**storage).appResRef);
			RemoveRsrcMap((**sStorage).rsrcMap);

	}
	
	RestoreA4();
	
	return result;
}
#endif /* RIBGENPLUGIN_TEST */

