#include <ri.h>
#include <stdio.h>
#include "RibGenPlugIn.h"
#include "InterfaceUtils.h"
#include <GDeviceStuff.h>

#define kCurrentDataVersion		0x00010000	/* version 1.0 */

typedef struct {
	long	dataVersion;
	float	rmax;
	float	zmin;
	float	zmax;
	float	thetamax;
} ParaboloidData, **ParaboloidDataH;

static ParaboloidData theData;

static OSErr 	HToParams(Handle h);
static OSErr 	ParamsToH(Handle h);
static Boolean	DoInterface(void);

/************************* STANDARD PLUGIN ROUTINES *************************/

ComponentResult OpenPlugIn(void)
{
	return RG_NO_ERR;
}

void ClosePlugIn(void)
{
}

short InitPlugIn(Handle plugInData)
{
	return RG_NO_ERR;
}

void CleanupPlugIn(Handle plugInData)
{
}

short ModifyPlugInObject(Handle plugInData, Handle object)
{
	OSErr		err;
	
	if ((err = HToParams(object)) != noErr)
		return RG_BAD_OBJECT;

	if( !DoInterface() )
		return RG_USER_CANCELLED;	/* dialog canceled */
	
	return ParamsToH(object);
}

static short sRibGenErrorCode;

void HandleRibGenError(long code, long severity, char *message)
{
	if (sRibGenErrorCode == RG_NO_ERR) {
		if (code == RIE_NOMEM)
			sRibGenErrorCode = RG_MEM_FULL;
		else if (code == RIE_DISKFULL)
			sRibGenErrorCode = RG_DISK_FULL;
		else
			sRibGenErrorCode = RG_RIBGEN_ERR;
	}
}


short GeneratePlugInRib(Handle plugInData, Handle object, float time)
{
	if (HToParams(object) != noErr)
		return RG_BAD_OBJECT;

	sRibGenErrorCode = RG_NO_ERR;
	
	RiWorldBegin();
	RiRotate(-90, 1, 0, 0);
	RiParaboloid(theData.rmax,
				 	theData.zmin,
				 	theData.zmax,
				 	theData.thetamax, RI_NULL);
	RiWorldEnd();

	return sRibGenErrorCode;
}

/************************* UTILITY ROUTINES *************************/

static OSErr HToParams(Handle object)
{
	short n;
	
	if (object == NULL)
		return RG_BAD_OBJECT;
	
	if (GetHandleSize(object) == 0
	  || (**(ParaboloidDataH)object).dataVersion != kCurrentDataVersion) {
		theData.rmax = 1.0;
		theData.zmin = 0.0;
		theData.zmax = 1.0;
		theData.thetamax = 360;
	}
	else {
		HLock(object);
		theData = **(ParaboloidDataH)object;
		HUnlock(object);
	}
	return RG_NO_ERR;
}

static OSErr ParamsToH(Handle object)
{
	if (object == NULL)
		return RG_BAD_OBJECT;
	
	SetHandleSize(object, sizeof(ParaboloidData) );
	if (MemError())
		return RG_MEM_FULL;
	theData.dataVersion = kCurrentDataVersion;
	**(ParaboloidDataH)object = theData;
	return RG_NO_ERR;
}

/************************* INTERFACE *************************/

enum
{
	diOk = 1,
	diCancel,
	diPicture,
	diTitle,
	diArg1,
	diArg2,
	diArg3,
	diArg4,
	diArg5,
	diArg6,
	diArg7
} dialogItem;

static Boolean
DoInterface(void)
{
	DialogPtr	dialog;
	short		hit;
	char		str[64];
	short		type;
	Handle		arg1Item, arg2Item, arg3Item, arg4Item, arg5Item, arg6Item, arg7Item;
	Rect		box;
	
	PositionDialogOnMain('DLOG', 128);
	dialog = GetNewDialog(128, NULL, (WindowPtr)-1L);
	if (dialog == NULL)
		return FALSE;

	sprintf( (char *) str, "%5.2f", theData.rmax );
	GetDItem(dialog, diArg1, &type, &arg1Item, &box );
	SetIText(arg1Item, c2pstr(str));

	sprintf( (char *) str, "%5.2f", theData.zmin );
	GetDItem(dialog, diArg2, &type, &arg2Item, &box );
	SetIText(arg2Item, c2pstr(str));

	sprintf( (char *) str, "%5.2f", theData.zmax );
	GetDItem(dialog, diArg3, &type, &arg3Item, &box );
	SetIText(arg3Item, c2pstr(str));

	sprintf( (char *) str, "%5.2f", theData.thetamax );
	GetDItem(dialog, diArg4, &type, &arg4Item, &box );
	SetIText(arg4Item, c2pstr(str));
	SelIText(dialog, diArg1, 0, 32767);

	ShowWindow(dialog);
	SelectWindow(dialog);

retry:
	do {
		ModalDialog(NULL, &hit);
	} while (hit != diOk && hit != diCancel);
	if (hit == diOk)
	{
		if( -1 == GetFloatValue( arg1Item, &theData.rmax, 0, 100000.0) )
		{
			SelIText(dialog, diArg1, 0, 32767);
			goto retry;
		}	
		if( -1 == GetFloatValue( arg2Item, &theData.zmin, -100000.0, 100000.0) )
		{
			SelIText(dialog, diArg2, 0, 32767);
			goto retry;
		}	
		if( -1 == GetFloatValue( arg3Item, &theData.zmax, theData.zmin, 1000000) )
		{
			SelIText(dialog, diArg3, 0, 32767);
			goto retry;
		}
		if( -1 == GetFloatValue( arg4Item, &theData.thetamax, 0, 360) )
		{
			SelIText(dialog, diArg4, 0, 32767);
			goto retry;
		}
	}
	DisposeDialog(dialog);
	return (hit == diOk);
}

